@@ -1,8 +1,7 @@
 /*
  * gif2png.c
  * Copyright (C) 1995 Alexander Lehmann
- * Bug fixes and enhancements by Greg Roelofs, 5 September 1999.
- * Production release packaging by Eric S. Raymond <esr@thyrsus.com>.
+ * Copyright (C) 1999 by Eric S. Raymond.
  * For conditions of distribution and use, see the file COPYING.
  */
 
@@ -10,11 +9,10 @@
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>	/* for isatty() */
-
-#if !defined(TRUE)
-#define FALSE	0
-#define TRUE	1
-#endif
+#include <zlib.h>
+#include <sys/stat.h>
+#include <utime.h>
+#include <stdbool.h>
 
 #include "gif2png.h"
 
@@ -23,30 +21,32 @@
 #  define png_jmpbuf(png_ptr) (png_ptr->jmpbuf)
 #endif
 
-struct GIFelement first={NULL};
+/*@i1@*/struct GIFelement first={NULL};
 struct GIFelement *current;
-int delete = FALSE;
-int optimize = FALSE;
-int histogram = FALSE;
-int interlaced = PNG_INTERLACE_LAST;
-int progress = FALSE;
-int verbose = FALSE;
-int recover = FALSE;
-int webconvert = FALSE;
-int software_chunk = TRUE;
-int filtermode = FALSE;
-int matte = FALSE;
-int gamma_srgb = FALSE;
-GifColor matte_color;
-long numgifs = 0L;
-long numpngs = 0L;
+bool recover = false;
+int verbose = 0;
 
+static bool delete = false;
+static bool optimize = false;
+static bool histogram = false;
+static int interlaced = PNG_INTERLACE_LAST;
+static bool preserve_mtime = false;
+static bool progress = false;
+static bool webconvert = false;
+static bool software_chunk = true;
+static bool filtermode = false;
+static bool matte = false;
+static bool gamma_srgb = false;
+static GifColor matte_color;
+static long numgifs = 0L;
+static long numpngs = 0L;
 
 int interlace_line(int height, int line)
 /* return the actual line # used to store an interlaced line */
 {
     int res;
 
+    /*@-shiftimplementation@*/
     if ((line & 7) == 0) {
 	return line >> 3;
     }
@@ -59,11 +59,13 @@ int interlace_line(int height, int line)
 	return res + ((line-2) >> 2);
     }
     return res + ((height+1) >> 2) + ((line-1) >> 1);
+    /*@=shiftimplementation@*/
 }
 
 int inv_interlace_line(int height, int line)
 /* inverse function of above, used for recovery of interlaced images */
 {
+    /*@-shiftimplementation@*/
     if ((line << 3)<height) {
 	return line << 3;
     }
@@ -77,6 +79,7 @@ int inv_interlace_line(int height, int line)
     }
     line -= (height + 1) >> 2;
     return (line << 1) + 1;
+    /*@=shiftimplementation@*/
 }
 
 /* 
@@ -102,7 +105,7 @@ int inv_interlace_line(int height, int line)
  * filtering, the 8-bit file will be larger.
  */
 
-int is_gray(png_color c)
+static bool is_gray(png_color c)
 {
     /* check if r==g==b, moved to function to work around NeXTstep 3.3 cc bug.
      * life could be so easy if at least the basic tools worked as expected.
@@ -110,7 +113,9 @@ int is_gray(png_color c)
     return c.red == c.green && c.green == c.blue;
 }
 
-int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
+/*@-globstate -mustfreefresh @*/
+static int writefile(struct GIFelement *s, struct GIFelement *e, 
+		     FILE *fp, bool lastimg)
 {
     int i;
     struct GIFimagestruct *img = e->imagestruct;
@@ -118,9 +123,9 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
     GifColor *colors = img->colors;
     int colors_used = 0;
     byte remap[MAXCMSIZE];
-    int low_prec;
-    png_struct *png_ptr = xalloc(sizeof (png_struct));
-    png_info *info_ptr = xalloc(sizeof (png_info));
+    bool low_prec;
+    png_struct *png_ptr;
+    png_info *info_ptr;
     int p;
     int gray_bitdepth;
     png_color pal_rgb[MAXCMSIZE], *pltep;
@@ -137,19 +142,39 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 
     /* these volatile declarations prevent gcc warnings ("variable might be
      *  clobbered by `longjmp' or `vfork'") */
-    volatile int gray = TRUE;
+    volatile bool gray = true;
     volatile int last_color = 0;
-    volatile int remapping = FALSE;
+    volatile bool remapping = false;
     volatile int bitdepth;
 
+    /*@-nullpass@*/
+    png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+    if (png_ptr == NULL) {
+	(void)fprintf(stderr, "gif2png: fatal error, out of memory\n");
+	(void)fprintf(stderr, "gif2png: exiting ungracefully\n");
+	exit(1);
+    }
+    info_ptr = png_create_info_struct(png_ptr);
+    if (info_ptr == NULL) {
+	(void)fprintf(stderr, "gif2png: fatal error, out of memory\n");
+	(void)fprintf(stderr, "gif2png: exiting ungracefully\n");
+	exit(1);
+    }
+    /*@=nullpass@*/
+
+    /* pacify splint by defining all this storage */
+    memset(remap, '\0', sizeof(remap));
+    memset(pal_rgb, '\0', sizeof(pal_rgb));
+    memset(pal_trans, '\0', sizeof(pal_trans));
+
     /* ignore the transparency value if no pixel in the image uses it */
-    if (img->trans != -1 && !count[img->trans])
+    if (img->trans != -1 && count[img->trans] == 0)
 	img->trans = -1;
 
     if (optimize) {
 	/* zero out unused color table entries and use zlib level 9 */
 	for (i = 0; i < MAXCMSIZE; i++)
-	    if(!count[i])
+	    if(count[i] == 0)
 		colors[i].red = colors[i].green = colors[i].blue = 0;
 	/*
 	 * The way that last_color is computed ensures that trailing zeroed-out
@@ -162,20 +187,20 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 	int unused = 0;
 
 	for (i = 0; i < MAXCMSIZE; i++)
-	    if (!count[i])
+	    if (count[i] == 0)
 		unused++;
 
-	if (unused)
+	if (unused > 0)
 	{
-	    fprintf(stderr, 
-		    "gif2png: %d unused colors; convert with -O to remove\n",
-		    unused);
+	    (void)fprintf(stderr, 
+			  "gif2png: %d unused colors; convert with -O to remove\n",
+			  unused);
 	    errorcount++;
 	}
     }
 
     for (i = 0; i < MAXCMSIZE; i++)
-	if (count[i]) {
+	if (count[i] != 0) {
 	    gray &= is_gray(colors[i]);
 	    colors_used++;
 	    if (last_color < i) 
@@ -184,15 +209,15 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 
     if (gray) {
 	if (verbose > 1)
-	    fprintf(stderr, "gif2png: input image is grayscale\n");
+	    (void)fprintf(stderr, "gif2png: input image is grayscale\n");
 
 	if (img->trans != -1) {
 	    for (i = 0; i < MAXCMSIZE; i++) {
 		if (i!=img->trans && colors[i].red == colors[img->trans].red) {
-		    gray = FALSE;
+		    gray = false;
 		    if (verbose > 1)
-			fprintf(stderr,
-				"gif2png: transparent color is repeated in visible colors, using palette\n");
+			(void)fprintf(stderr,
+				      "gif2png: transparent color is repeated in visible colors, using palette\n");
 		    break;
 		}
 	    }
@@ -205,7 +230,7 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
     if (last_color<2) bitdepth = 1;
 
     if (gray) {
-	remapping = TRUE;
+	remapping = true;
 	for (i = 0; i < MAXCMSIZE; i++)
 	    remap[i] = colors[i].red;
 
@@ -213,10 +238,10 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 
 	/* try to adjust to 4-bit precision grayscale */
 
-	low_prec = TRUE;
+	low_prec = true;
 	for (i = 0; i < MAXCMSIZE; i++) {
 	    if ((remap[i]&0xf)*0x11 != remap[i]) {
-		low_prec = FALSE;
+		low_prec = false;
 		break;
 	    }
 	}
@@ -232,7 +257,7 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 	if (low_prec) {
 	    for (i = 0; i < MAXCMSIZE; i++) {
 		if ((remap[i]&3)*5 != remap[i]) {
-		    low_prec = FALSE;
+		    low_prec = false;
 		    break;
 		}
 	    }
@@ -250,7 +275,7 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 	if (low_prec) {
 	    for (i = 0; i < MAXCMSIZE; i++) {
 		if ((remap[i]&1)*3 != remap[i]) {
-		    low_prec = FALSE;
+		    low_prec = false;
 		    break;
 		}
 	    }
@@ -263,40 +288,41 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 	}
 
 	if (bitdepth<gray_bitdepth) {
-	    gray = FALSE;		/* write palette file */
-	    remapping = FALSE;
+	    gray = false;		/* write palette file */
+	    remapping = false;
 	} else {
 	    bitdepth = gray_bitdepth;
 	}
     }
 
     if (verbose > 1)
-	fprintf(stderr, "gif2png: %d colors used, highest color %d, %s, bitdepth %d\n",
-		colors_used, last_color, gray ? "gray":"palette", bitdepth);
+	(void)fprintf(stderr, "gif2png: %d colors used, highest color %d, %s, bitdepth %d\n",
+		      colors_used, last_color, gray ? "gray":"palette", bitdepth);
 
     if (verbose > 2)
 	for (i = 0; i <= last_color; i++) {
-	    fprintf(stderr, "(%3d, %3d, %3d)\n", 
-		    colors[i].red, colors[i].blue, colors[i].green);  
+	    (void)fprintf(stderr, "(%3d, %3d, %3d)\n", 
+			  colors[i].red, colors[i].blue, colors[i].green);  
 	}
 
-    if (setjmp(png_jmpbuf(png_ptr))) {
-	fprintf(stderr, "gif2png:  libpng returns fatal error condition\n");
-	free(png_ptr);
-	free(info_ptr);
+    if (setjmp(png_jmpbuf(png_ptr)) != 0) {
+	(void)fprintf(stderr, "gif2png:  libpng returns fatal error condition\n");
+	(void)free(png_ptr);
+	(void)free(info_ptr);
 	return 1;
     }
 
-
     /* Create and initialize the png_struct with the desired error handler
      * functions.  If you want to use the default stderr and longjump method,
      * you can supply NULL for the last three parameters.  We also check that
      * the library version is compatible with the one used at compile time,
      * in case we are using dynamically linked libraries.  REQUIRED.
      */
+    /*@-nullpass@*/
     png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
 				      (void *)NULL, NULL, NULL);
 
+    /*@=nullpass@*/
     if (png_ptr == NULL)
 	return(2);
 
@@ -304,12 +330,12 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
     info_ptr = png_create_info_struct(png_ptr);
     if (info_ptr == NULL)
     {
-	png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
+	/*@i1@*/png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
 	return(2);
     }
 
     /* if errtype is not 1, this was generated by fatal() */ 
-    if ((errtype = setjmp(png_jmpbuf(png_ptr)))) {
+    if ((errtype = setjmp(png_jmpbuf(png_ptr))) != 0) {
 	png_destroy_write_struct(&png_ptr, &info_ptr);
 	return errtype;
     }
@@ -330,7 +356,7 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 
     /* set basic image properties: width/height, type, bit depth, interlace */
     if (interlaced == PNG_INTERLACE_LAST)
-	interlaced = current->imagestruct->interlace ? PNG_INTERLACE_ADAM7 : PNG_INTERLACE_NONE;
+	interlaced = current->imagestruct->interlace!=0 ? PNG_INTERLACE_ADAM7 : PNG_INTERLACE_NONE;
 
     png_set_IHDR(png_ptr, info_ptr, 
 		 current->imagestruct->width, current->imagestruct->height,
@@ -356,8 +382,10 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 
 	if (gray) {
 	    if (is_gray(GifScreen.ColorMap[GifScreen.Background])) {
-			color16back.gray = remap[GifScreen.Background];
+		color16back.gray = remap[GifScreen.Background];
+		/*@-compdef@*/
 		png_set_bKGD(png_ptr, info_ptr, &color16back);
+		/*@=compdef@*/
 	    }
 	} else {
 	    for (i = 0; i < MAXCMSIZE; i++) {
@@ -366,8 +394,10 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 		    GifScreen.ColorMap[GifScreen.Background].blue == colors[i].blue) {
 		    if (last_color < i) 
 			last_color = i;
-		    color16back.index = i;
+		    color16back.index = (png_byte)i;
+		    /*@-compdef@*/
 		    png_set_bKGD(png_ptr, info_ptr, &color16back);
+		    /*@=compdef@*/
 		    break;
 		}
 	    }
@@ -378,20 +408,26 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
     if (img->trans != -1) {
 	if (gray) {
 	    if (verbose > 2)
-		fprintf(stderr, "gif2png: handling grayscale transparency\n");
+		(void)fprintf(stderr, 
+			      "gif2png: handling grayscale transparency\n");
 	    color16trans.gray = remap[img->trans];
+	    /*@-nullpass -compdef@*/
 	    png_set_tRNS(png_ptr, info_ptr, NULL, 0, &color16trans);
+	    /*@=nullpass =compdef@*/
 	} else {
 	    if (verbose > 2)
-		fprintf(stderr, "gif2png: handling palette transparency\n");
+		(void)fprintf(stderr, 
+			      "gif2png: handling palette transparency\n");
 	    /* GRR NOTE: may be inconsistent with background-color logic (above) */
-	    remapping = TRUE;
+	    remapping = true;
 	    for (i = 0; i < MAXCMSIZE; i++)
-		remap[i] = i;
-	    remap[0] = img->trans;
+		remap[i] = (byte)i;
+	    remap[0] = (byte)img->trans;
 	    remap[img->trans] = 0;
 	    pal_trans[0] = 0;
+	    /*@-nullpass@*/
 	    png_set_tRNS(png_ptr, info_ptr, pal_trans, 1, NULL);
+	    /*@=nullpass@*/
 	}
     }
 
@@ -399,7 +435,7 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
     if (!gray) {
 	if (remapping) {
 	    if (verbose > 2)
-		fprintf(stderr, "Remapping palette...\n");
+		(void)fprintf(stderr, "Remapping palette...\n");
 	    for (i = 0; i <= last_color; i++) {
 		pal_rgb[i].red   = current->imagestruct->colors[remap[i]].red;
 		pal_rgb[i].green = current->imagestruct->colors[remap[i]].green;
@@ -408,16 +444,18 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 	    pltep = pal_rgb;
 	} else {
 	    if (verbose > 2)
-		fprintf(stderr, "Palette copied from GIF colors...\n");
+		(void)fprintf(stderr, "Palette copied from GIF colors...\n");
 	    pltep = current->imagestruct->colors;
 	}
 	png_set_PLTE(png_ptr, info_ptr, pltep, last_color+1);
 
 	if (verbose > 2) {
-	    fprintf(stderr,"PNG palette generated with %d colors:\n",last_color+1);
+	    (void)fprintf(stderr,
+			  "PNG palette generated with %d colors:\n",
+			  last_color+1);
 	    for (i = 0; i <= last_color; i++) {
-		fprintf(stderr, "(%3d, %3d, %3d)\n", 
-			pltep[i].red, pltep[i].blue, pltep[i].green);  
+		(void)fprintf(stderr, "(%3d, %3d, %3d)\n", 
+			      pltep[i].red, pltep[i].blue, pltep[i].green);  
 	    }
 	}
     }
@@ -434,7 +472,7 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 		histogr[i] = (png_uint_16)count[i];
 	} else {
 	    for (i = 0; i < MAXCMSIZE; i++)
-		if (count[i]) {
+		if (count[i] != 0) {
 		    histogr[i] = (png_uint_16)
 #ifdef __GO32__
 			/* avoid using fpu instructions on djgpp, so we don't need emu387 */
@@ -452,6 +490,7 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
     }
 
     /* transcribe software chunk if necessary */
+    /*@-observertrans -statictrans@ -unqualifiedtrans -branchstate*/
     if (software_chunk) {
 	software.compression = PNG_TEXT_COMPRESSION_NONE;
 	software.key = "Software";
@@ -460,6 +499,7 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 
 	png_set_text(png_ptr, info_ptr, &software, 1);
     }
+    /*@=observertrans =statictrans =unqualifiedtrans =branchstate@*/
 
     png_write_info(png_ptr, info_ptr);
 
@@ -472,24 +512,25 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
     while(lastimg ? s != NULL : s != e->next) {
 	byte *data;
 	switch((byte)s->GIFtype) {
+        /*@-branchstate@*/
 	case GIFimage:
 	    passcount = png_set_interlace_handling(png_ptr);
 	    for (p = 0; p<passcount;p++)
-		for (i = 0; i<current->imagestruct->height; i++) {
+		for (i = 0; i < (int)current->imagestruct->height; i++) {
 		    if (progress) {
 			if (passcount>1)
-			    fprintf(stderr, "%d/%d ", p+1, passcount);
-			fprintf(stderr, "%2d%%\r",
-				(int)(((long)i*100)/current->imagestruct->height));
-			fflush(stderr);
+			    (void)fprintf(stderr, "%d/%d ", p+1, passcount);
+			(void)fprintf(stderr, "%2d%%\r",
+				      (int)(((long)i*100)/current->imagestruct->height));
+			(void)fflush(stderr);
 		    }
 		    data = current->data + 
-				       (long) (img->interlace ?
-							interlace_line(current->imagestruct->height,i) : i) * current->imagestruct->width;
+				       (long) (img->interlace!=0 ?
+					       interlace_line((int)current->imagestruct->height,i) : i) * current->imagestruct->width;
 		    /* we have to remap once */
 		    if (remapping && p == 0)
 		    {
-			for (j = 0; j < img->width; j++) {
+			for (j = 0; j < (int)img->width; j++) {
 			    data[j] = remap[data[j]];
 			}
 		    }
@@ -498,24 +539,30 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 	    break;
 
 	case GIFcomment:
-	    j = s->size;
+	    j = (int)s->size;
 	    /* some GIF encoders produce trailing garbage and NULs */
-	    if (j > 0 && ((s->data[j-1] == '\0') || (s->data[j-1] & 0x80)))
+	    if (j > 0 && ((s->data[j-1] == '\0') || (s->data[j-1] & 0x80)!=0))
 		--j;
+	    /*@-observertrans -statictrans @*/
 	    comment.key = "Comment";
 	    if (j<500) {
 		comment.compression = PNG_TEXT_COMPRESSION_NONE;
 	    } else {
 		comment.compression = PNG_TEXT_COMPRESSION_zTXt;
 	    }
- 	    comment.text = (png_charp)s->data;
+	    comment.text = (png_charp)s->data;
  	    comment.text_length = strlen(comment.text);
  	    png_set_text(png_ptr, info_ptr, &comment, 1);
+	    /*@=observertrans =statictrans@*/
 	    break;
 
 	case GIFapplication:
-	    png_write_chunk_start(png_ptr, (unsigned char *)"gIFx", s->size);
-	    png_write_chunk_data(png_ptr, s->data, s->size);
+	    png_write_chunk_start(png_ptr, 
+				  (unsigned char *)"gIFx", 
+				  (png_uint_32)s->size);
+	    png_write_chunk_data(png_ptr, 
+				 s->data, 
+				 (png_size_t)s->size);
 	    png_write_chunk_end(png_ptr);
 	    break;
 
@@ -561,9 +608,12 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 	    break;
 
 	default:
-	    fprintf(stderr, "gif2png internal error: encountered unused element type %c\n", s->GIFtype);
+	    (void)fprintf(stderr, 
+			  "gif2png internal error: encountered unused element type %c\n", 
+			  s->GIFtype);
 	    break;
 	}
+        /*@=branchstate@*/
 	s = s->next;
     }
     png_write_end(png_ptr, info_ptr);
@@ -573,7 +623,9 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 
     return errorcount;
 }
+/*@=globstate =mustfreefresh@*/
 
+/*@-onlytrans@*/
 int
 MatteGIF(GifColor matte)
 /* convert transparent pixels to a matte of given color */
@@ -584,12 +636,12 @@ MatteGIF(GifColor matte)
     for (current = &first; current ; current = current->next) {
 	if (current->imagestruct) {
 	    if (current->imagestruct->trans == -1)
-		fprintf(stderr, 
+		(void)fprintf(stderr, 
 			"gif2png: no transparency color in image %d, matte argument ignored\n",
 			icount);
 	    else {
-		if (verbose) {
-		    fprintf(stderr, 
+		if (verbose > 0) {
+		    (void)fprintf(stderr, 
 			"gif2png: transparent value in image %d is %d\n",  
 			icount, current->imagestruct->trans);
 		}
@@ -601,30 +653,33 @@ MatteGIF(GifColor matte)
     }
     return icount;
 }
+/*@=onlytrans@*/
 
-int processfilter(void)
+/*@-onlytrans@*/
+static int processfilter(void)
 {
     int num_pics;
     struct GIFelement *start;
 
     current = &first;
     num_pics = ReadGIF (stdin);
-    fclose(stdin);
+    (void)fclose(stdin);
 
     if (num_pics != 1) {
-	fprintf(stderr, "gif2png: %d images in -f (filter) mode\n", num_pics);
+	(void)fprintf(stderr, 
+		      "gif2png: %d images in -f (filter) mode\n", num_pics);
 	return 1;
     }
 
     /* eliminate use of transparency, if that is called for */
     if (matte)
-	MatteGIF(matte_color);
+	(void)MatteGIF(matte_color);
 
     start = NULL;
     for (current = first.next; current ; current = current->next) {
 	if (start == NULL) start = current;
 	if (current->GIFtype == GIFimage) {
-	    (void)writefile(start, current, stdout, TRUE);
+	    (void)writefile(start, current, stdout, true);
 	    start = NULL;
 	    ++numpngs;
 	}
@@ -632,14 +687,18 @@ int processfilter(void)
     free_mem();
     return 0;
 }
+/*@=onlytrans@*/
 
-int processfile(char *fname, FILE *fp)
+/*@-onlytrans@*/
+static int processfile(char *fname, FILE *fp)
 {
     char outname[1025]; /* MAXPATHLEN comes under too many names */
     int num_pics;
     struct GIFelement *start;
-    int i, suppress_delete = FALSE;
+    int i; 
+    bool suppress_delete = false;
     char *file_ext;
+    struct utimbuf time_buf;
 
     if (fp == NULL) return 1;
 
@@ -647,10 +706,29 @@ int processfile(char *fname, FILE *fp)
 
     num_pics = ReadGIF (fp);
 
-    fclose(fp);
+    if (preserve_mtime)
+    {
+        struct stat stat_buf;
+
+	/*@-compdef@*/
+	/*@-unrecog@*//* needed because splint's POSIX library is buggy */
+        if (fstat(fileno(fp), &stat_buf) < 0)
+        {
+            perror("Could not get input file times");
+            return 1;
+        }
+	/*@=compdef@*/
+
+	/*@-usedef@*/
+        time_buf.actime  = stat_buf.st_atime;
+        time_buf.modtime = stat_buf.st_mtime;
+	/*@=usedef@*/
+    }
+
+    (void)fclose(fp);
 
     if (num_pics >= 0 && verbose > 1)
-	fprintf(stderr, "gif2png: number of images %d\n", num_pics);
+	(void)fprintf(stderr, "gif2png: number of images %d\n", num_pics);
 
     if (num_pics <= 0)
 	return 1;
@@ -658,24 +736,27 @@ int processfile(char *fname, FILE *fp)
     if (webconvert)
 	if (num_pics != 1)
 	{
-	    fprintf(stderr, "gif2png: %s is multi-image\n", fname);
+	    (void)fprintf(stderr, "gif2png: %s is multi-image\n", fname);
 	    return 0;
 	}
 	else
 	{
-	    printf("%s\n", fname);
+	    (void)printf("%s\n", fname);
 	    return 0;
 	}
 
     /* eliminate use of transparency, if that is called for */
     if (matte)
-	MatteGIF(matte_color);
+	(void)MatteGIF(matte_color);
 
     /* create output filename */
 
-    strcpy(outname, fname);
+    (void)strcpy(outname, fname);
+
+    file_ext = outname+strlen(outname);
+    if (file_ext >= outname + 4)
+	file_ext -= 4;
 
-    file_ext = outname+strlen(outname)-4;
     if (strcmp(file_ext, ".gif") != 0 && strcmp(file_ext, ".GIF") != 0 &&
 	strcmp(file_ext, "_gif") != 0 && strcmp(file_ext, "_GIF") != 0) {
 	/* try to derive basename */
@@ -690,7 +771,7 @@ int processfile(char *fname, FILE *fp)
 	}
     }
 
-    strcpy(file_ext, ".png"); /* images are named .png, .p01, .p02, ... */
+    (void)strcpy(file_ext, ".png"); /* images are named .png, .p01, .p02, ... */
 
     start = NULL;
 
@@ -705,10 +786,24 @@ int processfile(char *fname, FILE *fp)
 		return 1;
 	    } else {
 		suppress_delete |= writefile(start, current, fp, i == num_pics);
-		fclose(fp);
+		(void)fclose(fp);
+
+                if (preserve_mtime)
+                {
+		    /*@-compdef@*/
+                    if (utime(outname, &time_buf) < 0)
+                    {
+                        perror("Could not set output file times");
+                        return 1;
+                    }
+		    /*@=compdef@*/
+                }
+
 		++numpngs;
 		start = NULL;
-		sprintf(file_ext, ".p%02d", i);
+		/*@-bufferoverflowhigh*/
+		(void)sprintf(file_ext, ".p%02d", i);
+		/*@=bufferoverflowhigh*/
 	    }
 	}
     }
@@ -716,90 +811,94 @@ int processfile(char *fname, FILE *fp)
     free_mem();
 
     if (delete && !suppress_delete) {
-	unlink(fname);
+	(void)unlink(fname);
     }
 
     return 0;
 }
+/*@=onlytrans@*/
 
 /* check if stdin is a terminal, if your compiler is lacking isatty or fileno
-   (non-ANSI functions), just return 0
+   (non-ANSI functions), just return false
 */
 
-int input_is_terminal(void)
+static bool input_is_terminal(void)
 {
-    return isatty(fileno(stdin));
+    return isatty(fileno(stdin))!=0;
 }
 
 int main(int argc, char *argv[])
 {
     FILE *fp;
-    int i, background;
+    int i; 
+    unsigned int background;
     int errors = 0;
     char name[1025];
     int ac;
     char *color;
 
-    software_chunk = 1;
+    software_chunk = true;
 
-    ac = 1;
-
-    for (ac = 1; ac<argc && argv[ac][0] == '-'; ac++)
+    for (ac = 1; ac < argc && argv[ac][0] == '-'; ac++)
     {
-	for (i = 1;i<strlen(argv[ac]); i++)
+	for (i = 1; i < (int)strlen(argv[ac]); i++)
 	    switch(argv[ac][i]) {
 	    case 'b':
 		if (ac >= argc - 1)
 		{
-		    fputs("gif2png: missing background-matte argument\n",
-			  stderr);
+		    (void)fputs("gif2png: missing background-matte argument\n",
+				stderr);
 		    exit(1);
 		}
 		color = argv[++ac];
 		if (*color == '#')
 		    color++;
-	        sscanf(color, "%x", &background);
+	        (void)sscanf(color, "%ux", &background);
 		matte_color.red = (background >> 16) & 0xff;
 		matte_color.green = (background >> 8) & 0xff; 
 		matte_color.blue = background & 0xff;
-		matte = TRUE;
+		matte = true;
 		goto skiparg;
 
 	    case 'd':
-		delete = TRUE;
+		delete = true;
 		break;
 
 	    case 'f':
 	    case '1': /* backward compatibility */
-		filtermode = TRUE;
+		filtermode = true;
 		break;
 
 	    case 'g':
-		gamma_srgb = TRUE;
+		gamma_srgb = true;
 		break;
 
 	    case 'h':
-		histogram = TRUE;
+		histogram = true;
 		break;
 
 	    case 'i':
 		interlaced = PNG_INTERLACE_ADAM7;
 		break;
 
+	    case 'm':
+		preserve_mtime = true;
+		break;
+
 	    case 'n':
 		interlaced = PNG_INTERLACE_NONE;
 		break;
 
 	    case 'p':
-		progress = TRUE;
+		progress = true;
 		break;
 
 	    case 'r':
-		recover = TRUE;
+		recover = true;
 		break;
 
 	    case 's':
-		software_chunk = FALSE;
+		software_chunk = false;
 		break;
 
 	    case 'v':
@@ -807,17 +906,18 @@ int main(int argc, char *argv[])
 		break;
 
 	    case 'w':
-		webconvert = TRUE;
+		webconvert = true;
 		break;
 
 	    case 'O':
-		optimize = TRUE;
+		optimize = true;
 		break;
 
 	    default:
-		fprintf(stderr, "gif2png: unknown option `-%c'\n", argv[ac][i]);
+		(void)fprintf(stderr, 
+			      "gif2png: unknown option `-%c'\n", argv[ac][i]);
 	    usage:
-		fprintf(stderr,
+		(void)fprintf(stderr,
 	"\n"
 	"usage: gif2png [ -bdfghinprstvwO ] [file[.gif]] ...\n"
 	"\n"
@@ -827,6 +927,7 @@ int main(int argc, char *argv[])
 	"   -g  write gamma=(1.0/2.2) and sRGB chunks\n"
 	"   -h  generate PNG histogram chunks into color output files\n"
 	"   -i  force conversion to interlaced PNG files\n"
+	"   -m  preserve input file modification time\n"
 	"   -n  force conversion to noninterlaced PNG files\n"
 	"   -p  display progress of PNG writing in %%\n"
 	"   -r  try to recover corrupted GIF files\n"
@@ -846,7 +947,7 @@ int main(int argc, char *argv[])
     }
 
     if (verbose > 1)
-	fprintf(stderr, "gif2png: %s, %s\n", version, compile_info);
+	(void)fprintf(stderr, "gif2png: %s, %s\n", version, compile_info);
 
     /* loop over arguments or use stdin */
     if (argc == ac) {
@@ -854,19 +955,27 @@ int main(int argc, char *argv[])
 	    goto usage;
 	}
 	if (verbose > 1)
-	    fprintf(stderr, "stdin:\n");
+	    (void)fprintf(stderr, "stdin:\n");
 	if (filtermode) {
 	    if (processfilter() != 0) ++errors;
 	} else {
-	    processfile("noname.gif", stdin);
+	    (void)processfile("noname.gif", stdin);
 	    ++numgifs;
 	}
     } else {
 	for (i = ac;i<argc; i++) {
-	    strcpy(name, argv[i]);
+	    /* make sure that there is enough space for a '.p<NUM>' suffix;
+	       this check catches also the '.gif' case below. */
+	    if (strlen(argv[i]) >= sizeof name - sizeof ".p" - 3 * sizeof(int)) {
+		(void)fprintf(stderr, "%s: name too long\n", argv[i]);
+		errors = 1;
+		continue;
+	    }
+
+	    (void)strcpy(name, argv[i]);
 	    if ((fp = fopen(name, "rb")) == NULL) {
 		/* retry with .gif appended */
-		strcat(name, ".gif");
+		(void)strcat(name, ".gif");
 		fp = fopen(name,"rb");
 	    }
 	    if (fp == NULL) {
@@ -874,7 +983,7 @@ int main(int argc, char *argv[])
 		errors = 1;
 	    } else {
 		if (verbose > 1)
-		    fprintf(stderr, "%s:\n", name);
+		    (void)fprintf(stderr, "%s:\n", name);
 		errors |= processfile(name, fp);
 		/* fp is closed in processfile */
 		++numgifs;
@@ -882,10 +991,10 @@ int main(int argc, char *argv[])
 	}
     }
 
-    if (verbose)
-	fprintf(stderr, "Done (%s).  Converted %ld GIF%s into %ld PNG%s.\n",
-		errors? "with one or more errors" : "no errors detected", 
-		numgifs, (numgifs == 1) ? "" : "s", numpngs, (numpngs == 1)? "" : "s");
+    if (verbose > 0)
+	(void)fprintf(stderr, "Done (%s).  Converted %ld GIF%s into %ld PNG%s.\n",
+		      errors!=0? "with one or more errors" : "no errors detected", 
+		      numgifs, (numgifs == 1) ? "" : "s", numpngs, (numpngs == 1)? "" : "s");
 
     return errors;
 }
