@@ -18,9 +18,6 @@
 const char version[] = "gif2png " VERSION;
 
 const char compile_info[]=
-#ifdef TMPFILE
-  "(TMPFILE) "
-#endif
 #if defined(PNG_LIBPNG_VER_STRING) && defined(ZLIB_VERSION)
   "compiled " __DATE__ " with libpng " PNG_LIBPNG_VER_STRING " and zlib " ZLIB_VERSION ".";
 #else
